"""
indi game engine - keyboard input
"""
import igeCore as core
from .event import EventType

class KeyEventCode:
    PRESSED = 0
    RELEASED = 1

class KeyCode:
    KEY_NOKEY = -1
    KEY_0 = 0
    KEY_1 = 1
    KEY_2 = 2
    KEY_3 = 3
    KEY_4 = 4
    KEY_5 = 5
    KEY_6 = 6
    KEY_7 = 7
    KEY_8 = 8
    KEY_9 = 9
    KEY_A = 10
    KEY_B = 11
    KEY_C = 12
    KEY_D = 13
    KEY_E = 14
    KEY_F = 15
    KEY_G = 16
    KEY_H = 17
    KEY_I = 18
    KEY_J = 19
    KEY_K = 20
    KEY_L = 21
    KEY_M = 22
    KEY_N = 23
    KEY_O = 24
    KEY_P = 25
    KEY_Q = 26
    KEY_R = 27
    KEY_S = 28
    KEY_T = 29
    KEY_U = 30
    KEY_V = 31
    KEY_W = 32
    KEY_X = 33
    KEY_Y = 34
    KEY_Z = 35
    KEY_UP = 36
    KEY_RIGHT = 37
    KEY_DOWN = 38
    KEY_LEFT= 39
    KEY_F1 = 40
    KEY_F2 = 41
    KEY_F3 = 42
    KEY_F4 = 43
    KEY_F5 = 44
    KEY_F6 = 45
    KEY_F7 = 46
    KEY_F8 = 47
    KEY_F9 = 48
    KEY_F10 = 49
    KEY_F11 = 50
    KEY_F12 = 51
    KEY_ESC = 52
    KEY_TAB = 53
    KEY_BACK = 54
    KEY_RETURN = 55
    KEY_KP0 = 56
    KEY_KP1 = 57
    KEY_KP2 = 58
    KEY_KP3 = 59
    KEY_KP4 = 60
    KEY_KP5 = 61
    KEY_KP6 = 62
    KEY_KP7 = 63
    KEY_KP8 = 64
    KEY_KP9 = 65
    KEY_KPPLUS = 66
    KEY_KPMINUS = 67
    KEY_KPDIV = 68
    KEY_KPMULT = 69
    KEY_KPENTER = 70
    KEY_KPPERIOD = 71
    KEY_PAUSE = 72
    KEY_SPACE = 73
    KEY_PLUS = 74
    KEY_MINUS = 75
    KEY_PERIOD = 76
    KEY_SLASH = 77
    KEY_HASH = 78
    KEY_EQUAL = 79
    KEY_QUOTE = 80
    KEY_BACKQUOTE = 81
    KEY_SEMICOLON = 82
    KEY_LEFTBRACKET = 83
    KEY_RIGHTBRACKET = 84
    KEY_BACKSLASH = 85
    KEY_COMMA = 86
    KEY_INSERT = 87
    KEY_DEL = 88
    KEY_HOME = 89
    KEY_END = 90
    KEY_PAGEUP = 91
    KEY_PAGEDOWN = 92
    KEY_LCTRL = 93
    KEY_RCTRL = 94
    KEY_LALT = 95
    KEY_RALT = 96
    KEY_LWIN = 97
    KEY_RWIN = 98
    KEY_LSHIFT = 99
    KEY_RSHIFT = 100
    KEY_CAPSLOCK = 101
    KEY_NUMLOCK = 102
    KEY_COUNT = 103

class KeyModifier:
    KEY_MOD_NONE = 0
    KEY_MOD_CTRL = 1
    KEY_MOD_ALT = 2
    KEY_MOD_SHIFT = 4

class Keyboard:
    __instance = None

    def __init__(self):
        self.keyPressedCallbacks = set([])
        self.keyReleasedCallbacks = set([])

    def __del__(self):
        self.keyPressedCallbacks.clear()
        self.keyReleasedCallbacks.clear()

    @staticmethod
    def instance():
        if Keyboard.__instance is None:
            Keyboard.__instance = Keyboard()
        return Keyboard.__instance
     
    def getKey(self, keyCode):
        return core.getKeyChar(keyCode)

    def getModifier(self):
        return core.getKeyModifier()

    def isPressed(self, keyCode):
        return core.isKeyPressed(keyCode)

    def isReleased(self, keyCode):
        return core.isKeyReleased(keyCode)
        
    def isHold(self, keyCode):
        return core.isKeyHold(keyCode)

    def registerKeyPressedCallback(self, onKeyPressedFunc):
        self.keyPressedCallbacks.add(onKeyPressedFunc)
        if len(self.keyPressedCallbacks) == 1:
            core.registerEventListener(EventType.KEYBOARD, KeyEventCode.PRESSED, self.onKeyPressed)

    def registerKeyReleasedCallback(self,onKeyReleasedFunc):
        self.keyReleasedCallbacks.add(onKeyReleasedFunc)
        if len(self.keyReleasedCallbacks) == 1:
            core.registerEventListener(EventType.KEYBOARD, KeyEventCode.RELEASED, self.onKeyReleased)
    
    def removeKeyPressedCallback(self, onKeyPressedFunc):
        self.keyPressedCallbacks.remove(onKeyPressedFunc)

    def removeKeyReleasedCallback(self, onKeyReleasedFunc):
        self.keyReleasedCallbacks.remove(onKeyReleasedFunc)

    def onKeyPressed(self, keyCode, eventCode, keyChar, keyModifier):
        for callback in self.keyPressedCallbacks:
            callback(keyCode, eventCode, keyChar, keyModifier)

    def onKeyReleased(self, keyCode, eventCode, keyChar, keyModifier):
        for callback in self.keyReleasedCallbacks:
            callback(keyCode, eventCode, keyChar, keyModifier)



"""
graphicsHelper.py

Functions to help build graphics data
"""

import igeCore as core
import igeVmath as vmath

_uvs = (0.0, 1.0, 1.0, 1.0, 0.0, 0.0, 1.0, 0.0)
_tris = (0, 2, 1, 1, 2, 3)
_pivotoffset = (1,-1, 0,-1, -1,-1, 1,0, 0,0, -1,0, 1,1, 0,1, -1,1)


def textFigure(word, fontpath, fontsize,color=(1.0, 1.0, 1.0, 1.0), pivot=4, scale = 1.0):
    '''
    Create Visible text

    Parameters
    ----------
        word : string
            text
        fontpath : string
            font data file path
        fontsize : string
            font size
        color : (float,float,float,float)
            text color
        pivot : int
            center point of polygon
            0       1       2
             +-------+-------+
             |       |       |
            3|      4|      5|
             +-------+-------+
             |       |       |
            6|      7|      8|
             +-------+-------+
    '''

    w,h = core.calcFontPixelSize(word, fontpath, fontsize)
    tex = core.texture(core.unique("text"),w,h, format=core.GL_RED)
    tex.setText(word, fontpath,fontsize)

    gen = core.shaderGenerator()
    gen.setColorTexture(True)
    gen.setBoneCondition(1, 1)
    gen.discardColorMapRGB()

    hw = w/2 * scale
    hh = h/2 * scale
    px = _pivotoffset[pivot*2+0] * hw;
    py = _pivotoffset[pivot*2+1] * hh;

    points = ((-hw+px,hh+py,0.0), (hw+px,hh+py,0.0), (-hw+px,-hh+py,0.0), (hw+px,-hh+py,0.0))

    efig = createMesh(points, _tris, None, _uvs, gen)

    efig.setMaterialParam("mate", "DiffuseColor", color);
    efig.setMaterialParamTexture("mate", "ColorSampler", tex,
                                    wrap_s=core.SAMPLERSTATE_BORDER,wrap_t=core.SAMPLERSTATE_BORDER,
                                    minfilter=core.SAMPLERSTATE_NEAREST, magfilter=core.SAMPLERSTATE_LINEAR)
    efig.setMaterialRenderState("mate", "blend_enable", True)

    return efig


def createSprite(width:float=100, height:float=100, texture=None, uv_left_top:tuple=(0,0), uv_right_bottom:tuple=(1,1), normal=None, pivot=4):
    """
    Create Visible Rectangle

	Parameters
	----------
        width : float (optional)
            sprite width
        height: float (optional)
            sprite height
        texture: string (optional)
            texture file name
        uv_left_top: tuple (optional)
            texture uv value of left top cornar
        uv_right_bottom: tuple (optional)
            texture uv value of right bottom cornar
        normal: pyvmath.vec3 (optional)
            specify rectangle's nomal vector
        pivot : int
            center point of polygon
	Returns
	-------
        editableFigure
    """
    hw = width/2
    hh = height/2
    px = _pivotoffset[pivot*2+0] * hw;
    py = _pivotoffset[pivot*2+1] * hh;
    points = ((-hw+px,hh+py,0.0), (hw+px,hh+py,0.0), (-hw+px,-hh+py,0.0), (hw+px,-hh+py,0.0))

    if normal is not None:
        newpoints = []
        nom0 = (0, 0, 1)
        mat = vmath.mat33(vmath.quat_rotation(nom0, normal))
        for p in points:
            newpoints.append(mat * p)
        points = newpoints

    # uvs = (uv_left_top[0], uv_right_bottom[1],
    #        uv_right_bottom[0], uv_right_bottom[1],
    #        uv_left_top[0], uv_left_top[1],
    #        uv_right_bottom[1], uv_left_top[1])
    uvs = ( uv_left_top[0], uv_right_bottom[1],
            uv_right_bottom[0], uv_right_bottom[1],
            uv_left_top[0], uv_left_top[1],
            uv_right_bottom[0], uv_left_top[1])

    return createMesh(points, _tris, texture, uvs)


def createMesh(points, tris, texture=None, uvs = None, shader = None, normals = None):
    """
    Create a polygon mesh by specifying vertex coordinates and triangle index

	Parameters
	----------
        points: tuple or list
            list or tuple of points
        tris: tuple or list
            list or tuple of triangle indices
        texture: string (optional)
            file path of texture
        uvs: list or tuple (optional)
            list or tuple of  texture uv
        normals : list or tuple (optional)
            list or tuple of  vertex normal

	Returns
	-------
        editableFigure
   
    """
    if shader is None:
        shader = core.shaderGenerator()
        if texture != None:
            shader.setColorTexture(True)
        shader.setBoneCondition(1, 1)

    efig = core.editableFigure("sprite")
    efig.addMaterial("mate", shader)
    efig.addMesh("mesh", "mate");

    efig.setVertexElements("mesh", core.ATTRIBUTE_ID_POSITION, points)
    if uvs:
        efig.setVertexElements("mesh", core.ATTRIBUTE_ID_UV0,uvs)

    if  normals:
        efig.setVertexElements("mesh", core.ATTRIBUTE_ID_NORMAL,normals)

    efig.setTriangles("mesh", tris);
    efig.addJoint("joint");
    efig.setMaterialParam("mate", "DiffuseColor", (1.0, 1.0, 1.0, 1.0));
    #efig.setMaterialRenderState("mate", "cull_face_enable", False)

    if texture != None:
        efig.setMaterialParamTexture("mate", "ColorSampler", texture,
                                     wrap_s=core.SAMPLERSTATE_BORDER,wrap_t=core.SAMPLERSTATE_BORDER,
                                     minfilter=core.SAMPLERSTATE_LINEAR, magfilter=core.SAMPLERSTATE_LINEAR)
        efig.setMaterialRenderState("mate", "blend_enable", True)

    return efig



"""
indi game engine - touch input
"""
import igeCore as core
from .event import EventType

class TouchEventCode:
    BEGAN = 0
    MOVED = 1
    ENDED = 2
    SCROLLED = 3

class Touch:
    __instance = None

    def __init__(self):
        self.touchBeganCallbacks = set([])
        self.touchMovedCallbacks = set([])
        self.touchEndedCallbacks = set([])
        self.touchScrolledCallbacks = set([])

    def __del__(self):
        self.touchBeganCallbacks.clear()
        self.touchMovedCallbacks.clear()
        self.touchEndedCallbacks.clear()
        self.touchScrolledCallbacks.clear()

    @staticmethod
    def instance():
        if Touch.__instance is None:
            Touch.__instance = Touch()
        return Touch.__instance

    def getFingerPosition(self, fingerId):
        return core.getFingerPosition(fingerId)

    def isFingerPressed(self, fingerId):
        return core.isFingerPressed(fingerId)

    def isFingerMoved(self, fingerId):
        return core.isFingerMoved(fingerId)

    def isFingerReleased(self, fingerId):
        return core.isFingerReleased(fingerId)

    def isFingerScrolled(self, fingerId):
        return core.isFingerScrolled(fingerId)

    def getFingerScrolledData(self, fingerId):
        return core.getFingerScrolledData(fingerId)

    def getFingerPressure(self, fingerId):
        return core.getFingerPressure(fingerId)

    def getFingersCount(self):
        return core.getFingersCount()

    def getAllFingers(self):
        return core.getAllFingers()

    def registerTouchBeganCallback(self, onTouchBeganFunc):
        self.touchBeganCallbacks.add(onTouchBeganFunc)
        if len(self.touchBeganCallbacks) == 1:
            core.registerEventListener(EventType.TOUCH, TouchEventCode.BEGAN, self.onTouchBegan)

    def registerTouchMovedCallback(self, onTouchMovedFunc):
        self.touchMovedCallbacks.add(onTouchMovedFunc)
        if len(self.touchMovedCallbacks) == 1:
            core.registerEventListener(EventType.TOUCH, TouchEventCode.MOVED, self.onTouchMoved)

    def registerTouchEndedCallback(self, onTouchEndedFunc):
        self.touchEndedCallbacks.add(onTouchEndedFunc)
        if len(self.touchEndedCallbacks) == 1:
            core.registerEventListener(EventType.TOUCH, TouchEventCode.ENDED, self.onTouchEnded)

    def registerTouchScrolledCallback(self, onTouchScrolledFunc):
        self.touchScrolledCallbacks.add(onTouchScrolledFunc)
        if len(self.touchScrolledCallbacks) == 1:
            core.registerEventListener(EventType.TOUCH, TouchEventCode.SCROLLED, self.onTouchScrolled)

    def removeTouchBeganCallback(self, onTouchBeganFunc):
        self.touchBeganCallbacks.remove(onTouchBeganFunc)

    def removeTouchMovedCallback(self, onTouchMovedFunc):
        self.touchMovedCallbacks.remove(onTouchMovedFunc)

    def removeTouchEndedCallback(self, onTouchEndedFunc):
        self.touchEndedCallbacks.remove(onTouchEndedFunc)

    def removeTouchScrolledCallback(self, onTouchScrolledFunc):
        self.touchScrolledCallbacks.remove(onTouchScrolledFunc)

    def onTouchBegan(self, id, x, y, pressure):
        for callback in self.touchBeganCallbacks:
            callback(id, x, y, pressure)

    def onTouchMoved(self, id, x, y, pressure):
        for callback in self.touchMovedCallbacks:
            callback(id, x, y, pressure)

    def onTouchEnded(self, id, x, y, pressure):
        for callback in self.touchEndedCallbacks:
            callback(id, x, y, pressure)

    def onTouchScrolled(self, id, scroll_x, scroll_y, is_inverse):
        for callback in self.touchScrolledCallbacks:
            callback(id, scroll_x, scroll_y, is_inverse)



"""
indi game engine
"""

from igeCore._igeCore import *

SAMPLERSTATE_WRAP = 0
SAMPLERSTATE_MIRROR = 1
SAMPLERSTATE_CLAMP = 2
SAMPLERSTATE_BORDER = 3
SAMPLERSTATE_LINEAR = 0
SAMPLERSTATE_NEAREST = 1
SAMPLERSTATE_NEAREST_MIPMAP_NEAREST = 2
SAMPLERSTATE_LINEAR_MIPMAP_NEAREST = 3
SAMPLERSTATE_NEAREST_MIPMAP_LINEAR = 4
SAMPLERSTATE_LINEAR_MIPMAP_LINEAR = 5

TARGET_PLATFORM_PC = 0
TARGET_PLATFORM_IOS = 1
TARGET_PLATFORM_ANDROID = 2
TARGET_PLATFORM_MOBILE= 3
CURRENT_PLATFORM = getPlatform()

ANIMETION_PART_A = 0
ANIMETION_PART_B = 1
ANIMETION_PART_C = 2

ANIMETION_SLOT_A0 = 1
ANIMETION_SLOT_A1 = 2
ANIMETION_SLOT_B0 = 3
ANIMETION_SLOT_B1 = 4
ANIMETION_SLOT_C0 = 5
ANIMETION_SLOT_C1 = 6

ATTRIBUTE_ID_POSITION = 1
ATTRIBUTE_ID_NORMAL = 2
ATTRIBUTE_ID_TANGENT = 3
ATTRIBUTE_ID_BINORMAL = 4
ATTRIBUTE_ID_UV0 = 5
ATTRIBUTE_ID_UV1 = 6
ATTRIBUTE_ID_UV2 = 7
ATTRIBUTE_ID_UV3 = 8
ATTRIBUTE_ID_COLOR = 9
ATTRIBUTE_ID_BLENDINDICES = 10
ATTRIBUTE_ID_BLENDWEIGHT = 11
ATTRIBUTE_ID_PSIZE = 12

GL_BYTE            =0x1400
GL_UNSIGNED_BYTE   =0x1401
GL_SHORT           =0x1402
GL_UNSIGNED_SHORT  =0x1403
GL_INT             =0x1404
GL_UNSIGNED_INT    =0x1405
GL_FLOAT           =0x1406
GL_2_BYTES         =0x1407
GL_3_BYTES         =0x1408
GL_4_BYTES         =0x1409
GL_DOUBLE          =0x140A
GL_HALF_FLOAT      =0x140B

GL_RED             =0x1903
GL_RGB             =0x1907
GL_RGBA            =0x1908

MAPCHANNEL_NONE = 0
MAPCHANNEL_COLOR_RED = 1
MAPCHANNEL_COLOR_ALPHA = 2
MAPCHANNEL_NORMAL_RED = 3
MAPCHANNEL_NORMAL_ALPHA = 4
MAPCHANNEL_LIGHT_RED = 5
MAPCHANNEL_LIGHT_ALPHA = 6
MAPCHANNEL_VERTEX_COLOR_RED = 7
MAPCHANNEL_VERTEX_COLOR_ALPHA = 8

AMBIENT_TYPE_NONE = 0
AMBIENT_TYPE_AMBIENT = 1
AMBIENT_TYPE_HEMISPHERE = 2

HAPTIC_SELECTION = 0
HAPTIC_SUCCESS = 1
HAPTIC_WARNING = 2
HAPTIC_FAILURE = 3
HAPTIC_LIGHT_IMPACT = 4
HAPTIC_MEDIUM_IMPACT = 5
HAPTIC_HEAVY_IMPACT = 6

ADS_BANNER_TOP = 0
ADS_BANNER_BOTTOM = 1
ADS_BANNER_TOPLEFT = 2
ADS_BANNER_TOPRIGHT = 3
ADS_BANNER_BOTTOMLEFT = 4
ADS_BANNER_BOTTOMRIGHT = 5

Quality_Fastest = 0
Quality_Normal = 1
Quality_Production = 2
Quality_Highest = 3


LocalSpace = 0
WorldSpace = 1



import imgui
import igeCore as core
import igeVmath as vmath
import igeCore.input as input

class ImgiIGERenderer(object):
    def __init__(self):
        if not imgui.get_current_context():
            raise RuntimeError(
                "No valid ImGui context. Use imgui.create_context() first and/or "
                "imgui.set_current_context()."
            )
        self.io = imgui.get_io()
        self._font_texture = None
        self.editableFigure = None
        self.camera = None
        self.showcase = None
        self._create_device_objects()
        self.refresh_font_texture()
        self.touch = input.getTouch()
        self.touch.registerTouchBeganCallback(self.onTouchBegan)
        self.touch.registerTouchMovedCallback(self.onTouchMoved)
        self.touch.registerTouchEndedCallback(self.onTouchEnded)
        self.touch.registerTouchScrolledCallback(self.onTouchScrolled)

    def render(self, draw_data, clearColor = True):

        io = self.io
        display_width, display_height = io.display_size
        fb_width = int(display_width * io.display_fb_scale[0])
        fb_height = int(display_height * io.display_fb_scale[1])
        if fb_width == 0 or fb_height == 0:
            return
        draw_data.scale_clip_rects(*io.display_fb_scale)

        self.editableFigure.clearMesh()

        idx = 0
        for commands in draw_data.commands_lists:
            idx_buffer_offset = 0
            mesh_name = "m"+str(idx)
            self.editableFigure.addMesh(mesh_name, "mate01")

            #Vertex format description
            #(ID, size, normalize, type)
            attr = ((core.ATTRIBUTE_ID_POSITION,2,False,core.GL_FLOAT),
                    (core.ATTRIBUTE_ID_UV0,2,False,core.GL_FLOAT),
                    (core.ATTRIBUTE_ID_COLOR,4,True,core.GL_UNSIGNED_BYTE))
            self.editableFigure.setVertexPtr(mesh_name, commands.vtx_buffer_data, commands.vtx_buffer_size,attr)
            self.editableFigure.setTrianglePtr(mesh_name, commands.idx_buffer_data, commands.idx_buffer_size//3)

            drawset=0
            for command in commands.commands:
                self.editableFigure.addDrawSet(mesh_name, idx_buffer_offset//3, command.elem_count//3)
                self.editableFigure.setDrawSetRenderState(mesh_name,drawset, "scissor_test_enable", True)
                x, y, z, w = command.clip_rect
                self.editableFigure.setDrawSetRenderState(mesh_name, drawset, 'scissor', int(x), int(fb_height - w), int(z - x), int(w - y))
                self.editableFigure.setDrawSetRenderState(mesh_name,drawset, "blend_enable", True)
                self.editableFigure.setDrawSetRenderState(mesh_name,drawset, "cull_face_enable", False)
                self.editableFigure.setDrawSetRenderState(mesh_name,drawset, "depth_test_enable", False)

                idx_buffer_offset += command.elem_count * imgui.INDEX_SIZE
                drawset+=1
            idx+=1

        self.camera.shoot(self.showcase, clearColor=clearColor)

    def refresh_font_texture(self):
        w, h, pixels = self.io.fonts.get_tex_data_as_alpha8()
        fonttexture = core.texture("fonttex", w, h, core.GL_RED, False, False, pixels)
        self.editableFigure.setMaterialParamTexture("mate01", "ColorSampler", fonttexture,
                                                    minfilter=core.SAMPLERSTATE_LINEAR, mipfilter=core.SAMPLERSTATE_LINEAR)

    def _create_device_objects(self):
        self.camera = core.camera('2dcamera')
        self.camera.orthographicProjection = True
        self.camera.position = vmath.vec3(0,0,1)
        self.camera.screenScale = (1,-1)
        w,h = core.viewSize();
        self.camera.screenOffset = -w , h

        gen = core.shaderGenerator()
        gen.setColorTexture(True)
        gen.setVertexColor(True)
        gen.discardColorMapRGB()
        self.editableFigure = core.editableFigure('font')
        self.editableFigure.addMaterial("mate01", gen)
        self.editableFigure.setMaterialParam("mate01", "DiffuseColor", (1.0, 1.0, 1.0, 1.0))
        self.editableFigure.setMaterialRenderState("mate01", "blend_enable", True)
        self.editableFigure.addJoint('joint')
        self.showcase = core.showcase('imgui')
        self.showcase.add(self.editableFigure)

    def _invalidate_device_objects(self):
        print('_invalidate_device_objects')

    def shutdown(self):
        self._invalidate_device_objects()

    def process_inputs(self):
        self.io.display_size = core.viewSize()
        w,h = core.viewSize()
        self.camera.screenOffset = -w , h

        # touch = core.singleTouch()
        # if touch != None:
            # curX = touch['cur_x'] + w//2
            # curY = -touch['cur_y'] + h//2
            # self.io.mouse_pos = curX, curY
            # self.io.mouse_down[0] = touch['is_holded']|touch['is_moved']

    def onTouchBegan(self, id, x, y, pressure):
        w,h = core.viewSize()
        curX = x + w//2
        curY = -y + h//2
        self.io.mouse_pos = curX, curY
        
        if id == 0:
            self.io.mouse_down[0] = True
        if id == 2:
            self.io.mouse_down[1] = True

    def onTouchMoved(self, id, x, y, pressure):
        w,h = core.viewSize()
        curX = x + w//2
        curY = -y + h//2
        self.io.mouse_pos = curX, curY

    def onTouchScrolled(self, id, x, y, is_inverse):
        if is_inverse:
            y *= -1
        self.io.mouse_wheel = y
    
    def onTouchEnded(self, id, x, y, pressure):
        if id == 0:
            self.io.mouse_down[0] = False
        if id == 2:
            self.io.mouse_down[1] = False


import importlib
try:
    importlib.import_module('numpy')
except ImportError:
    from pip._internal import main as _main
    _main(['install', 'numpy'])

from setuptools import setup, Extension, find_packages
import setuptools
import numpy
import sys
import os
from distutils.sysconfig import get_python_lib
import shutil

is64Bit = sys.maxsize > 2 ** 32
bindir = ''
if is64Bit:
    bindir = 'bin/win64'
else:
    bindir = 'bin/win32'

shutil.copy2(bindir+'/pyxcore.dll', 'igeCore')
shutil.copy2(bindir+'/pyxtools.dll', 'igeCore/devtool')
shutil.copy2(bindir+'/PVRTexLib.dll', 'igeCore/devtool')

pyxie_module = Extension('igeCore._igeCore', 
                       sources=[
                           'pythonEnvironment.cpp',
                           'pyxieFile.cpp',
                           'pythonEditableFigure.cpp',
                           'Window.cpp',
                           'pythonShowcase.cpp',
                           'pythonAnimator.cpp',
                           'pythonModule.cpp',
                           'pythonCamera.cpp',
                           'pythonShaderGenerator.cpp',
                           'pythonFigure.cpp',
                           'pythonResource.cpp',
                           'pythonTexture.cpp',
                           'pythonParticle.cpp',
                           'bitmapHelper.cpp',
                           'pythonProfiler.cpp',
                           'pythonHaptic.cpp',
                           'pythonInput.cpp'
                       ],
                       include_dirs=['bin/include', numpy.get_include()],
                       library_dirs=[bindir],
                       libraries=['pyxcore', 'user32', 'Gdi32'],
                       extra_compile_args=['/std:c++17'])

tools_module = Extension('igeCore.devtool._igeTools', 
                       sources=['pythonTools.cpp'],
                       include_dirs=['bin/include', numpy.get_include()],
                       library_dirs=[bindir],
                       libraries=['pyxtools','pyxcore'])

setup(name='igeCore', version='0.4.10',
		description='indi game engine core module',
		author=u'Indigames',
		author_email='dev@indigames.net',
		packages=find_packages(),
		ext_modules=[pyxie_module, tools_module],
		long_description=open('README.md').read(),
		license='MIT',
		install_requires=['igeVmath', 'requests', 'numpy'],
		classifiers=[
			'Intended Audience :: Developers',
			'License :: OSI Approved :: MIT License',
			'Programming Language :: Python :: 3',
			#'Operating System :: MacOS :: MacOS X',
			#'Operating System :: POSIX :: Linux',
			'Operating System :: Microsoft :: Windows',
			'Topic :: Games/Entertainment',
		],
        #data_files=[
        #    ('Lib/site-packages/pyxie', [bindir+"/pyxcore.dll"]),
        #    ('Lib/site-packages/pyxie/devtool',  [bindir+"/pyxtools.dll", bindir+"/PVRTexLib.dll"])
        #],
        package_data={'igeCore': ['*.dll', 'devtool/*.dll']},        
        include_package_data=True,
        setup_requires=['wheel']
      )
